using System;
using System.IO;
using System.Text;
using System.Web;
using System.Web.Mvc;
using MrCMS.Entities.People;
using MrCMS.Website;
using Newtonsoft.Json;

namespace MrCMS.Web.Areas.Admin.Controllers
{
    [MrCMSAuthorize(Roles = UserRole.Administrator)]
    [ValidateInput(false)]
    public abstract class AdminController : Controller
    {
        public new HttpRequestBase Request
        {
            get { return RequestMock ?? base.Request; }
        }

        public HttpRequestBase RequestMock { get; set; }

        protected override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            ViewData["controller-name"] = ControllerContext.RouteData.Values["controller"];
            base.OnActionExecuting(filterContext);
        }

        protected override JsonResult Json(object data, string contentType, Encoding contentEncoding,
                                           JsonRequestBehavior behavior)
        {
            return base.Json(data, contentType, contentEncoding, JsonRequestBehavior.AllowGet);
        }

        protected override JsonResult Json(object data, string contentType, Encoding contentEncoding)
        {
            return new JsonNetResult
            {
                ContentEncoding = contentEncoding,
                ContentType = contentType,
                JsonRequestBehavior = JsonRequestBehavior.AllowGet,
                Data = data
            };
        }
        protected JsonResult Json(string data)
        {
            return new JsonNetResult
            {
                ContentEncoding = null,
                ContentType = null,
                JsonRequestBehavior = JsonRequestBehavior.AllowGet,
                JsonData = data
            };
        }

        public class JsonNetResult : JsonResult
        {
            public JsonNetResult()
            {
                JsonRequestBehavior = JsonRequestBehavior.DenyGet;
            }

            public string JsonData { get; set; }

            public override void ExecuteResult(ControllerContext context)
            {
                if (context == null)
                    throw new ArgumentNullException("context");

                HttpResponseBase response = context.HttpContext.Response;

                response.ContentType = !String.IsNullOrEmpty(ContentType)
                                           ? ContentType
                                           : "application/json";
                if (ContentEncoding != null)
                    response.ContentEncoding = ContentEncoding;

                if (!string.IsNullOrWhiteSpace(JsonData))
                {
                    response.Write(JsonData);
                    return;
                }
                if (Data == null) return;

                var serializedData = JsonConvert.SerializeObject(Data);
                response.Write(serializedData);
            }
        }
    }
}